@@ -651,7 +651,7 @@ pkg_run_func_at(int dirfd, const char *vdb_path, const char *phases, const char
 		"debug-print-section() { :; }\n"
 		/* Not quite right */
 		"has_version() { [ -n \"$(qlist -ICqe \"$1\")\" ]; }\n"
-		/* best_version() */
+		"best_version() { qlist -ICqev \"$1\"; }\n"
 		"use() { useq \"$@\"; }\n"
 		"usex() { useq \"$1\" && echo \"${2-yes}$4\" || echo \"${3-no}$5\"; }\n"
 		"useq() { hasq \"$1\" ${USE}; }\n"
@@ -660,6 +660,7 @@ pkg_run_func_at(int dirfd, const char *vdb_path, const char *phases, const char
 		"hasq() { local h=$1; shift; case \" $* \" in *\" $h \"*) return 0;; *) return 1;; esac; }\n"
 		"hasv() { hasq \"$@\" && echo \"$1\"; }\n"
 		"elog() { printf ' * %%b\\n' \"$*\"; }\n"
+		"einfon() { printf ' * %%b' \"$*\"; }\n"
 		"einfo() { elog \"$@\"; }\n"
 		"ewarn() { elog \"$@\"; }\n"
 		"eqawarn() { elog \"QA: \"\"$@\"; }\n"
@@ -685,6 +686,9 @@ pkg_run_func_at(int dirfd, const char *vdb_path, const char *phases, const char
 		"D=\"%5$s\"\n"
 		"ED=\"${D%%/}/${EPREFIX%%/}/\"\n"
 		"T=\"%6$s\"\n"
+		/* we do not support preserve-libs yet, so force
+		 * preserve_old_lib instead */
+		"FEATURES=\"${FEATURES/preserve-libs/disabled}\"\n"
 		/* Finally run the func */
 		"%7$s%2$s\n"
 		/* Ignore func return values (not exit values) */
@@ -1780,7 +1784,8 @@ extern bool qlist_match(
 		tree_pkg_ctx *pkg_ctx,
 		const char *name,
 		depend_atom **name_atom,
-		bool exact);
+		bool exact,
+		bool applymasks);
 
 static int
 qmerge_unmerge_cb(tree_pkg_ctx *pkg_ctx, void *priv)
@@ -1797,7 +1802,7 @@ qmerge_unmerge_cb(tree_pkg_ctx *pkg_ctx, void *priv)
 
 	(void)list_set(priv, &todo);
 	for (p = todo; *p != NULL; p++) {
-		if (qlist_match(pkg_ctx, *p, NULL, true))
+		if (qlist_match(pkg_ctx, *p, NULL, true, false))
 			pkg_unmerge(pkg_ctx, NULL, cp_argc, cp_argv, cpm_argc, cpm_argv);
 	}
 
