@@ -21,8 +21,8 @@
 #include "atom.h"
 #include "basename.h"
 #include "contents.h"
+#include "hash.h"
 #include "human_readable.h"
-#include "md5_sha1_sum.h"
 #include "scandirat.h"
 #include "set.h"
 #include "tree.h"
@@ -167,19 +167,22 @@ check_pkg_install_mask(char *name)
 }
 
 static int
-qpkg_make(depend_atom *atom)
+qpkg_make(tree_pkg_ctx *pkg)
 {
-	FILE *fp, *out;
+	FILE *out;
+	FILE *fp;
 	char tmpdir[BUFSIZE];
 	char filelist[BUFSIZE + 32];
 	char tbz2[BUFSIZE + 32];
-	size_t buflen;
+	char buf[BUFSIZE * 4];
 	size_t xpaksize;
-	char *buf;
+	char *line;
+	char *savep;
 	int i;
 	char *xpak_argv[2];
 	struct stat st;
 	mode_t mask;
+	depend_atom *atom = tree_get_atom(pkg, false);
 
 	if (pretend) {
 		printf(" %s-%s %s:\n",
@@ -187,79 +190,58 @@ qpkg_make(depend_atom *atom)
 		return 0;
 	}
 
-	buflen = _Q_PATH_MAX;
-	buf = xmalloc(buflen);
-
-	snprintf(buf, buflen, "%s/%s/%s/CONTENTS",
-			portvdb, atom->CATEGORY, atom->PF);
-	if ((fp = fopen(buf, "r")) == NULL) {
-		free(buf);
+	line = tree_pkg_meta_get(pkg, CONTENTS);
+	if (line == NULL)
 		return -1;
-	}
 
 	snprintf(tmpdir, sizeof(tmpdir), "%s/qpkg.XXXXXX", qpkg_bindir);
 	mask = umask(0077);
 	i = mkstemp(tmpdir);
 	umask(mask);
-	if (i == -1) {
-		fclose(fp);
-		free(buf);
+	if (i == -1)
 		return -2;
-	}
 	close(i);
 	unlink(tmpdir);
-	if (mkdir(tmpdir, 0750)) {
-		fclose(fp);
-		free(buf);
+	if (mkdir(tmpdir, 0750))
 		return -3;
-	}
 
 	snprintf(filelist, sizeof(filelist), "%s/filelist", tmpdir);
-	if ((out = fopen(filelist, "w")) == NULL) {
-		fclose(fp);
-		free(buf);
+	if ((out = fopen(filelist, "w")) == NULL)
 		return -4;
-	}
 
-	while (getline(&buf, &buflen, fp) != -1) {
+	for (; (line = strtok_r(line, "\n", &savep)) != NULL; line = NULL) {
 		contents_entry *e;
-		e = contents_parse_line(buf);
+		e = contents_parse_line(line);
 		if (!e || e->type == CONTENTS_DIR)
 			continue;
 		if (check_pkg_install_mask(e->name) != 0)
 			continue;
-		fprintf(out, "%s\n", e->name+1); /* dont output leading / */
+		fprintf(out, "%s\n", e->name+1); /* don't output leading / */
 		if (e->type == CONTENTS_OBJ && verbose) {
-			char *hash = (char *)hash_file(e->name, HASH_MD5);
+			char *hash = hash_file(e->name, HASH_MD5);
 			if (hash != NULL) {
 				if (strcmp(e->digest, hash) != 0)
 					warn("MD5: mismatch expected %s got %s for %s",
 							e->digest, hash, e->name);
-				free(hash);
 			}
 		}
 	}
 
 	fclose(out);
-	fclose(fp);
 
 	printf(" %s-%s %s: ", GREEN, NORM,
 			atom_format("%[CATEGORY]%[PF]", atom));
 	fflush(stdout);
 
 	snprintf(tbz2, sizeof(tbz2), "%s/bin.tbz2", tmpdir);
-	if (snprintf(buf, buflen, "tar jcf '%s' --files-from='%s' "
-			"--no-recursion >/dev/null 2>&1", tbz2, filelist) > (int)buflen ||
-			(fp = popen(buf, "r")) == NULL)
-	{
-		free(buf);
+	if (snprintf(buf, sizeof(buf), "tar jcf '%s' --files-from='%s' "
+			"--no-recursion >/dev/null 2>&1", tbz2, filelist) >
+			(int)sizeof(buf) || (fp = popen(buf, "r")) == NULL)
 		return 2;
-	}
 	pclose(fp);
 
 	if ((i = open(tbz2, O_WRONLY)) < 0) {
 		warnp("failed to open '%s': %s", tbz2, strerror(errno));
-		free(buf);
 		return 1;
 	}
 
@@ -267,13 +249,12 @@ qpkg_make(depend_atom *atom)
 	if (fstat(i, &st) == -1) {
 		warnp("could not stat '%s': %s", tbz2, strerror(errno));
 		close(i);
-		free(buf);
 		return 1;
 	}
 	xpaksize = st.st_size;
 
-	snprintf(buf, buflen, "%s/%s/%s",
-			portvdb, atom->CATEGORY, atom->PF);
+	snprintf(buf, sizeof(buf), "%s%s/%s/%s",
+			portroot, portvdb, atom->CATEGORY, atom->PF);
 	xpak_argv[0] = buf;
 	xpak_argv[1] = NULL;
 	xpak_create(AT_FDCWD, tbz2, 1, xpak_argv, 1, verbose);
@@ -282,7 +263,6 @@ qpkg_make(depend_atom *atom)
 	if (fstat(i, &st) == -1) {
 		warnp("could not stat '%s': %s", tbz2, strerror(errno));
 		close(i);
-		free(buf);
 		return 1;
 	}
 	xpaksize = st.st_size - xpaksize;
@@ -291,7 +271,6 @@ qpkg_make(depend_atom *atom)
 	if ((fp = fdopen(i, "a")) == NULL) {
 		warnp("could not open '%s': %s", tbz2, strerror(errno));
 		close(i);
-		free(buf);
 		return 1;
 	}
 
@@ -303,14 +282,13 @@ qpkg_make(depend_atom *atom)
 	unlink(filelist);
 
 	/* create dirs, if necessary */
-	snprintf(buf, buflen, "%s/%s", qpkg_bindir, atom->CATEGORY);
+	snprintf(buf, sizeof(buf), "%s/%s", qpkg_bindir, atom->CATEGORY);
 	mkdir_p(buf, 0755);
 
-	snprintf(buf, buflen, "%s/%s/%s.tbz2",
+	snprintf(buf, sizeof(buf), "%s/%s/%s.tbz2",
 			qpkg_bindir, atom->CATEGORY, atom->PF);
 	if (rename(tbz2, buf)) {
 		warnp("could not move '%s' to '%s'", tbz2, buf);
-		free(buf);
 		return 1;
 	}
 
@@ -318,14 +296,12 @@ qpkg_make(depend_atom *atom)
 
 	if (stat(buf, &st) == -1) {
 		warnp("could not stat '%s': %s", buf, strerror(errno));
-		free(buf);
 		return 1;
 	}
 
 	printf("%s%s%s KiB\n",
 			RED, make_human_readable_str(st.st_size, 1, KILOBYTE), NORM);
 
-	free(buf);
 	return 0;
 }
 
@@ -334,7 +310,7 @@ qpkg_cb(tree_pkg_ctx *pkg, void *priv)
 {
 	size_t *pkgs_made = priv;
 
-	if (qpkg_make(tree_get_atom(pkg, false)) == 0)
+	if (qpkg_make(pkg) == 0)
 		(*pkgs_made)++;
 
 	return 0;
@@ -383,20 +359,27 @@ int qpkg_main(int argc, char **argv)
 			(fd != -1 && (fstat(fd, &st) == -1 || !S_ISDIR(st.st_mode))))
 	{
 		errp("could not create temp bindir '%s'", qpkg_bindir);
-	} else {
+	}
+	if (fd >= 0) {
 		/* fd is valid, pointing to a directory */
 		if (!restrict_chmod)
 			if (fchmod(fd, 0750) < 0)
 				errp("could not chmod(0750) temp bindir '%s'", qpkg_bindir);
-	}
-	if (fd >= 0)
 		close(fd);
+	}
 
 	/* we have to change to the root so that we can feed the full paths
 	 * to tar when we create the binary package. */
 	xchdir(portroot);
 
-	/* first process any arguments which point to /var/db/pkg */
+	ctx = tree_open_vdb(portroot, portvdb);
+	if (!ctx)
+		return EXIT_FAILURE;
+
+	/* first process any arguments which point to /var/db/pkg, an
+	 * undocumented method to allow easily tab-completing into vdb as
+	 * arguments, the trailing / needs to be present for this (as tab
+	 * completion would do) */
 	pkgs_made = 0;
 	s = strlen(portvdb);
 	for (i = optind; i < argc; ++i) {
@@ -405,41 +388,43 @@ int qpkg_main(int argc, char **argv)
 			argv[i] = NULL;
 			continue;
 		}
-		if (argv[i][asize-1] == '/')
-			argv[i][asize-1] = '\0';
-		if (!strncmp(portvdb, argv[i], s))
-			memmove(argv[i], argv[i]+s+1, asize-s);
-		else if (argv[i][0] == '/' && !strncmp(portvdb, argv[i]+1, s))
-			memmove(argv[i], argv[i]+s+2, asize-s-1);
-		else
-			continue;
+		if (asize > s && argv[i][0] == '/' && argv[i][asize - 1] == '/') {
+			char *path = argv[i];
 
-		atom = atom_explode(argv[i]);
-		if (atom) {
-			if (!qpkg_make(atom)) ++pkgs_made;
-			atom_implode(atom);
-		} else
-			warn("could not explode '%s'", argv[i]);
-		argv[i] = NULL;
+			/* chop off trailing / */
+			argv[i][asize - 1] = '\0';
+
+			/* eliminate duplicate leading /, we know it starts with / */
+			while (path[1] == '/')
+				path++;
+
+			if (strncmp(portvdb, path, s) == 0) {
+				path += s + 1 /* also eat / after portvdb */;
+				argv[i] = path;
+			} else {
+				argv[i][asize - 1] = '/';  /* restore, it may be a cat match */
+			}
+		}
 	}
 
 	/* now try to run through vdb and locate matches for user inputs */
-	ctx = tree_open_vdb(portroot, portvdb);
-	if (!ctx)
-		return EXIT_FAILURE;
-
 	for (i = optind; i < argc; ++i) {
 		if (argv[i] == NULL)
 			continue;
 		if (strcmp(argv[i], "world") == 0) {
-			/* we're basically done, this means all */
+			/* this is a crude hack, we include all packages for this,
+			 * which isn't exactly @world, but all its deps too */
 			tree_foreach_pkg_fast(ctx, qpkg_cb, &pkgs_made, NULL);
+			break;  /* no point in continuing since we did everything */
 		}
 		atom = atom_explode(argv[i]);
 		if (atom == NULL)
 			continue;
 
+		s = pkgs_made;
 		tree_foreach_pkg_fast(ctx, qpkg_cb, &pkgs_made, atom);
+		if (s == pkgs_made)
+			warn("no match for '%s'", argv[i]);
 		atom_implode(atom);
 	}
 	tree_close(ctx);
